/*
 * This file is part of gshutdown.
 *
 * Copyright (c) 2006-2009, GShutdown Team <gshutdown@gmail.com>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with This program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/*******************************************************************************
** Info: 
**    Parts of code are taken from examples of how to use libnotify correctly and 
**    at the same time comply to the new jaunty notification spec (read: visual 
**    guidelines)
** Copyright 2009 Canonical Ltd.
**
** Author:
**    Mirco "MacSlow" Mueller <mirco.mueller@canonical.com>
**
** This program is free software: you can redistribute it and/or modify it
** under the terms of the GNU General Public License version 3, as published
** by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranties of
** MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
** PURPOSE.  See the GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program.  If not, see <http://www.gnu.org/licenses/>.
**
** Authors:
**      Francesco Marella <francesco.marella@gmail.com>
*******************************************************************************/

#ifndef __GSHUTDOWN_GUI__
#define __GSHUTDOWN_GUI__

#include <gtk/gtk.h>
#include <glade/glade.h>
#ifdef LIBNOTIFY_ENABLED
#include <glib.h>
#include <unistd.h>
#include <math.h>
#include <libnotify/notify.h>
#endif
#include "values.h"
#include "gtkdateentry.h"

/* Variables {{{1 */

extern GladeXML *glade_xml;

/* main window */
extern GtkWidget *vbox_select_time;
extern GtkSpinButton *spin_hour;
extern GtkSpinButton *spin_minute;
extern GtkVBox *vbox_date;
extern GtkHBox *hbox_time;
extern GtkComboBox *combo_action;
extern GtkButton *btn_actual_time;
extern GtkButton *btn_profiles_show_hide;
extern GtkFrame *frame_profiles;
extern GtkButton *btn_start;
extern GtkDateEntry *date_entry;

/* others */
/* TODO : Rearange this ! */
extern GtkWidget *about_dialog;
extern GtkWidget *combo_method_manual_desktop;
extern GtkWidget *combo_method_manual_display_manager;
extern GtkWidget *radio_action_method_autodetect;
extern GtkWidget *radio_action_method_manual;
extern GtkWidget *radio_action_method_custom_command;
extern GtkWidget *check_command_before_action;
extern GtkWidget *method_manual_dialog;
extern GtkWidget *btn_action_method_manual;
extern GtkButton *btn_test_notifications;
extern GtkWidget *main_window;
extern GtkComboBox *combo_time_type;
extern GtkWidget *mnu_tray_cancel;
extern GtkStatusIcon *tray_icon;
extern GtkCheckButton *check_show_notifications;
extern GtkWidget *radio_systray_always;
extern GtkWidget *radio_systray_if_active;
extern GtkWidget *radio_systray_never;
extern GtkDialog *config_action_dialog;
extern GtkWidget *tray_eventbox;
extern GtkWidget *entry_action_custom_command;
extern GtkWidget *btn_config_shutdown;
extern GtkWidget *btn_config_restart;
extern GtkWidget *btn_config_logout;
extern GtkWidget *label_action_description;
extern GtkWidget *check_iconify;
extern GtkWidget *config_dialog;
extern GtkWidget *btn_check_system;
extern GtkWidget *entry_command_before_action;
extern GtkWidget *notification_date_dialog;
extern GtkComboBox *notification_date_combo;
extern GtkButton *notification_date_add;
extern GtkButton *notification_date_remove;
extern GtkSpinButton *spin_notification_hour, *spin_notification_minute, *spin_notification_second; 
extern GtkWidget *config_add_action_dialog;
extern GtkWidget *config_edit_action_dialog;
extern GtkComboBox *combo_add_action_dialog_type;
extern GtkWidget *entry_add_action_dialog_value;
extern GtkWidget *entry_add_action_dialog_name;
extern GtkComboBox *combo_edit_action_dialog_type;
extern GtkWidget *entry_edit_action_dialog_value;
extern GtkWidget *entry_edit_action_dialog_name;
extern GtkTreeView *actions_treeview;

/* }}}1 */

/* Prototypes {{{1 */

gboolean confirm_dialog(Action *id_action);
gboolean gui_init(int argc, char **argv);
gboolean gui_connect_all();
gboolean gui_systray_icon();
void gui_set_actual_time(gboolean, gboolean);
void gui_set_time(guint id);
GladeXML *gui_load_glade(gchar *file);
GtkMenu* traymenu();
GtkWidget* create_mnu_tray_action_now (void);
void gui_set_time(guint id);
void gui_systray_icon_hide();
void gui_systray_icon_show();
gboolean gui_systray_icon_is_showed();
gboolean timer_day();
gboolean gui_mark_today();
gboolean detect_error_dialog();
gboolean handle_arguments(int argc, char **argv);
void gui_remove_notification_date();
void gui_set_notification_date();

/* }}}1 */

typedef enum {
    NOTIFICATION_TYPE_IMPORTANT = 0,
    NOTIFICATION_TYPE_FAST_INFO = 1 
} NotificationDelay;

typedef enum {
    NOTIFICATION_ICON_NONE = 0,
    NOTIFICATION_ICON_TIME = 1 
} NotificationIcon;

typedef enum _Capability {
    CAP_ACTIONS = 0,
    CAP_BODY,
    CAP_BODY_HYPERLINKS,
    CAP_BODY_IMAGES,
    CAP_BODY_MARKUP,
    CAP_ICON_MULTI,
    CAP_ICON_STATIC,
    CAP_SOUND,
    CAP_IMAGE_SVG,
    CAP_SYNCHRONOUS,
    CAP_APPEND,
    CAP_LAYOUT_ICON_ONLY,
    CAP_MAX
} Capability;

#define ACTIONS          "actions"
#define BODY             "body"
#define BODY_HYPERLINKS  "body-hyperlinks"
#define BODY_IMAGES      "body-images"
#define BODY_MARKUP      "body-markup"
#define ICON_MULTI       "icon-multi"
#define ICON_STATIC      "icon-static"
#define SOUND            "sound"
#define IMAGE_SVG        "image/svg+xml"
#define SYNCHRONOUS      "private-synchronous"
#define APPEND           "append"
#define LAYOUT_ICON_ONLY "private-icon-only"

void sync_date_from_date_entry(GtkDateEntry *dateentry);

void gui_notification(gchar *message, NotificationDelay type_delay, NotificationIcon show_icon);

void gui_set_action_list();
void gui_set_action_list_combo();

#ifdef LIBNOTIFY_ENABLED
void closed_handler (NotifyNotification* notification, gpointer data);
void set_cap (gpointer data, gpointer user_data);
void init_caps (void);
gboolean has_cap (Capability cap);
static GdkPixbuf *scale_pixbuf_preserve_aspect_ratio (GdkPixbuf *pixbuf, gint size);
#endif

#endif

/* vim:cindent:et:sw=4:ts=4:sts=4:tw=78:fenc=utf-8:foldmethod=marker
 */
